import { UANodeSet } from "./UANodeSet";
import { UABaseNode } from "./UABaseNode";
import { NamespaceTable } from "./NameSpaceTable";

export class AddressSpace{

    nodeMap:  Map<string, UABaseNode>;

    constructor(public nodesets: UANodeSet[]) {
        this.reIndex(nodesets);
        this.nodeMap=this.buildNodeMap(nodesets);
        this.resolveChildren(nodesets, this.nodeMap);
    }

    private reIndex(nodesets:UANodeSet[]) {
        const nst=new NamespaceTable();
        for(const nodeset of nodesets) {
            nodeset.reIndex(nst);
        }
    }
    
    private buildNodeMap(nodesets:UANodeSet[]) {
        const nm=new Map<string, UABaseNode>();
        for(const nodeset of nodesets) {
            for(const node of nodeset.nodes) {
                nm.set(node.nodeId.toString(), node);
            }
        }
        return nm;
    }

    private resolveChildren(nodesets:UANodeSet[], nm:Map<string, UABaseNode>) {
        for(const nodeset of nodesets) {
            nodeset.resolveChildren(nm);
        }
    }

    public findNode(nodeId: string):UABaseNode|undefined {
        return this.nodeMap.get(nodeId)
    }

    static async load(files: string[]) {
        const sets:UANodeSet[]= [];
        for(const file of files) {
            const ns=await UANodeSet.load(file);
            sets.push(ns);
        }
        return new AddressSpace(sets);
    }
}