import type { NamespaceTable } from "./NameSpaceTable";
import { NodeId } from "./NodeId";
import { ReferencyType, UAReference } from "./UAReference";
import { assert } from "./util/assert";

export class UABaseNode {
    children: UABaseNode[]=[];
    constructor(public nodeId: NodeId,
                public browseName: string,
                public displayName: string,
                public references: UAReference[]) {
    }

    reIndex(nst: NamespaceTable, onst: NamespaceTable) {
        const nsName=onst.getUri(this.nodeId.namespace);
        assert(nsName!=undefined)
        const newIndex=nst.getIndex(nsName);
        assert(newIndex!=undefined)
        this.nodeId.namespace=newIndex;

        for(const uaref of this.references) {
            uaref.reIndex(nst, onst);
        }
    }

    resolveChildren(nm: Map<string, UABaseNode>) {
        for(const ref of this.references) {
            const node=nm.get(ref.ref.toString()) 
            if(node==undefined) {
                //TODO: parse all types/nodes
                continue;
            }
            if(ref.referenceType==ReferencyType.HasComponent && ref.isForward) {
                this.children.push(node);
            }
            if(ref.referenceType==ReferencyType.HasComponent && ref.isForward==false) {
                node.children.push(this);
            }
            if(ref.referenceType==ReferencyType.Organizes && ref.isForward) {
                this.children.push(node);
            }
            if(ref.referenceType==ReferencyType.Organizes && ref.isForward==false) {
                node.children.push(this);
            }
        }
    }

    static  parse(xmlObject: any): UABaseNode{
        const xmlReferences=xmlObject['References'];
        const references:UAReference[]=[];
        for(const xmlref of xmlReferences.Reference) {
            references.push(UAReference.parse(xmlref));
        }
        const ua=new UABaseNode(NodeId.coerceNodeId(xmlObject['@_NodeId']), xmlObject['@_BrowseName'], xmlObject['DisplayName'], references);
        return ua;
    }
}