import type { NamespaceTable } from "./NameSpaceTable";
import { NodeId } from "./NodeId";
import { assert } from "./util/assert";

export class UAReference {
    constructor(public referenceType: ReferencyType,
                public ref: NodeId,
                public isForward: boolean) {
    }

    reIndex(nst: NamespaceTable, onst: NamespaceTable) {
        const nsName=onst.getUri(this.ref.namespace);
        assert(nsName!=undefined)
        const newIndex=nst.getIndex(nsName);
        assert(newIndex!=undefined)
        this.ref.namespace=newIndex;    
    }

    static parse(uaReference: any): UAReference {
        return new UAReference(ReferencyType[uaReference['@_ReferenceType'] as keyof typeof ReferencyType], 
                                NodeId.coerceNodeId(uaReference['#text']), 
                                uaReference['@_IsForward']!="false");
    }
}


export enum ReferencyType {
    HasComponent  = 1,
    HasProperty =2,
    Organizes =3,
    HasOrderedComponent =4,
    HasModellingRule =5,
    HasTypeDefinition =6,
    HasEncoding =7,
    HasDescription =8,
    GeneratesEvent =9,
    AlwaysGeneratesEvent=10,
    HasNotifier=11
}